/*
 * Project name:
     Button R click
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     - initial release (DM)
     - mikroC for PIC (AM)
 * Description:
     This program demonstrates usage of the Button R click. In this example light
     on the button switch (6.8mm in diameter and has a red LED backlight)
     is controlled by PWM as well as TFT display backlight.
     When pressed, it sends an interrupt signal to the target board microcontroller.
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev. Board:      EasyPIC v7 Connectivity
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL, 8MHz Crystal, 32MHz MCU Clock
     ext. modules:    Button R Click board
                      http://www.mikroe.com/click/button-r/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
     - Place Button R click board in the mikroBUS socket 1.
     - Put jumpers SW4.1-SW4.4 in On position for 7-seg
     - Put jumper on J21 in RE1 position for Buzzer
*******************************************************************************/

#include "Display_Utils.h"

// Declare global variables
unsigned short current_duty, old_duty, current_duty1, old_duty1;
char tmp;
char i;
char j;
bit PWM_flag;
char Example_State=0;
unsigned int   digit;
unsigned short portd_array[4]={0};
unsigned short shifter, portd_index;

/*******************************************************************************
* Function Interrupt()
* ------------------------------------------------------------------------------
* Overview: External interrupt on Button press and Timer interrupt
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Interrupt() {
  // This is external INT0 interrupt (for sync start)

 if (INT0IF_bit) {
   Example_State++;                                    // set new ENC button state
   INT0IF_bit = 0;
   PWM_flag = 1;
  }
  
  // This is timer interrupt flag
 if (TMR0IF_bit) {
   LATA = 0;                                           // Turn off all 7seg displays
   LATD = portd_array[portd_index];                    // bring appropriate value to PORTD
   LATA = shifter;                                     // turn on appropriate 7seg. display

   // move shifter to next digit
   shifter <<= 1;
   if (shifter > 8u)
    shifter = 1;

   // increment portd_index
   portd_index++ ;
   if (portd_index > 3u)
    portd_index = 0;                                   // turn on 1st, turn off 2nd 7seg.
    TMR0IF_bit = 0;                                    // Clear TMR0IF
  }
}

/*******************************************************************************
* Function Sound_p()
* ------------------------------------------------------------------------------
* Overview: Function play sound on RE1 pin
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Sound_p(){
  TMR0IE_bit = 0;                                      // Disable timer interrupt
  Sound_Play(550, 50);                                 // Play sound at 550Hz for 50 miliseconds
  TMR0IE_bit = 1;                                      // Enable timer interrupt
}

/*******************************************************************************
* Function Countdown()
* ------------------------------------------------------------------------------
* Overview: Function countdown from 10 to 0 with sound
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Countdown(){
  digit=0;

 for(j=9;j>0;j--){
   digit = (j / 10u) % 10u;                            // extract tens digit
   portd_array[3] = mask(digit);                       // and store it to PORTD array
   delay_ms(7);
   digit = j % 10u;                                    // extract ones digit
   portd_array[2] = mask(digit);                       // and store it to PORTD array
   delay_ms(4);


   for(i=60;i>0;i--){
     digit = (i / 10u) % 10u;                          // extract tens digit
     portd_array[1] = mask(digit);                     // and store it to PORTD array
     delay_ms(7);
     digit = i % 10u;                                  // extract ones digit
     portd_array[0] = mask(digit);                     // and store it to PORTD array
     delay_ms(7);
    }
    Sound_p();
  }
   if(j>=0){

    portd_array[2] = mask(0x00);                       // and store it to PORTD array
    delay_ms(4);


    for(i=60;i>0;i--){
     digit = (i / 10u) % 10u;                          // extract tens digit
     portd_array[1] = mask(digit);                     // and store it to PORTD array
     delay_ms(7);
     digit = i % 10u;                                  // extract ones digit
     portd_array[0] = mask(digit);                     // and store it to PORTD array
     delay_ms(7);
    }
    Sound_p();
  }
  if(i>=0){
    portd_array[0] = mask(0x00);                       // and store it to PORTD array
    delay_ms(7);
  }
}

/*******************************************************************************
* Function Button_PWM()
* ------------------------------------------------------------------------------
* Overview: Function drives Button LED diode with PWM
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void Button_PWM(){
   // Button Fade In
  for(tmp=0;tmp<255;tmp++){
    current_duty++;
    PWM2_Set_Duty(current_duty);                       // Set current duty for PWM2
    Delay_ms(4);                                       // slow down change pace a little
   if(PWM_flag == 1){                                  // If Button is pressed leave function
     break;
    }
  }

   // Button Fade Out
 for(tmp=0;tmp<255;tmp++){
    current_duty--;
    PWM2_Set_Duty(current_duty);                       // Set current duty for PWM2
    Delay_ms(4);                                       // slow down change pace a little
   if(PWM_flag == 1){                                  // If Button is pressed leave function
     break;
    }
  }
}

/******************************************************************************
* Function InitMain()
//******************************************************************************/
void InitMain() {
  // Configure AN pins as digital
  ANSELA = 0;
  ANSELB = 0;
  ANSELC = 0;
  ANSELD = 0;
  ANSELE = 0;

  TRISA = 0;                         // Configure PORTA as input
  LATA  = 0;                         // Set PORTA to 0
  TRISB = 0x01;                      // Configure PORTB as output
  LATB  = 0;                         // Set PORTB to 0
  TRISC = 0;                         // Configure PORTC as output
  LATC  = 0;                         // Set PORTC to 0
  TRISD = 0;                         // Configure PORTD as output
  LATD  = 0;                         // Set PORTD to 0
  
  CCP2M2_bit = 1;
  CCP2M3_bit = 1;
  P2M1_bit  = 1;
  STR2B_bit = 1;
  PWM2_Init(5000);                   // Initialize PWM2 module at 5KHz
  Sound_Init(&PORTE, 1);             // Intitialize Sound pin on RE1

  T1CON = 0x21;
  T0CON = 0xC4;                      // Set TMR0 in 8bit mode, assign prescaler to TMR0
  TMR0L = 0;                         // Clear TMROL
  portd_index = 0;
  shifter = 1;
}

//******************************************************************************
// Main function
//******************************************************************************
void main() {

  InitMain();
  current_duty  = 0;                                   // Initial value for current_duty

  PWM2_Start();                                        // Start PWM2
  PWM2_Set_Duty(current_duty);                         // Set current duty for PWM2

  // Enable interrupts
  GIE_bit = 1 ;
  TMR0IE_bit = 1;
  TMR0IF_bit = 0;
  INT0IE_bit = 1 ;
  INT0IF_bit = 0 ;

  while (1) {                                          // Endless loop
  
    switch(Example_State){
     case 0 : {
            Button_PWM();                              // Fade In and Fade Out effect on Button LED diode
           }; break;

     case 1 : {
            PWM2_Stop();                               // Stop PWM on button
            Example_State=2;
           }; break;

     case 2 : {
            Countdown();                               // Start countdown
            Example_State = 3;
           }; break;

     case 3 : {
            Sound_p();                                 // Play sound
           }; break;

     case 4 : {
            PWM2_Start();                              // Start PWM2
            PWM2_Set_Duty(current_duty);               // Set current duty for PWM2
            Example_State = 0;
            PWM_flag = 0;                              // Clear PWM flag
            current_duty=0;
            LATA = 0;                                  // Turn off all 7seg displays
           };break;
    }
  }
}
/*******************************************************************************
* End of File
*******************************************************************************/